MockArquivos = [
    "DADOS GERAIS                : dger.dat\n",
    "DADOS DOS SUBSISTEMAS       : sistema.dat\n",
    "CONFIGURACAO HIDRAULICA     : confhd.dat\n",
    "ALTERACAO DADOS USINAS HIDRO: modif.dat\n",
    "CONFIGURACAO TERMICA        : conft.dat\n",
    "DADOS DAS USINAS TERMICAS   : term.dat\n",
    "DADOS DAS CLASSES TERMICAS  : clast.dat\n",
    "DADOS DE EXPANSAO HIDRAULICA: exph.dat\n",
    "ARQUIVO DE EXPANSAO TERMICA : expt.dat\n",
    "ARQUIVO DE PATAMARES MERCADO: patamar.dat\n",
    "ARQUIVO DE CORTES DE BENDERS: cortes.dat\n",
    "ARQUIVO DE CABECALHO CORTES : cortesh.dat\n",
    "RELATORIO DE CONVERGENCIA   : pmo.dat\n",
    "RELATORIO DE E. SINTETICAS  : parp.dat\n",
    "RELATORIO DETALHADO FORWARD : forward.dat\n",
    "ARQUIVO DE CABECALHO FORWARD: forwarh.dat\n",
    "ARQUIVO DE S.HISTORICAS S.F.: shist.dat\n",
    "ARQUIVO DE MANUT.PROG. UTE'S: manutt.dat\n",
    "ARQUIVO P/DESPACHO HIDROTERM: newdesp.dat\n",
    "ARQUIVO C/TEND. HIDROLOGICA : vazpast.dat\n",
    "ARQUIVO C/DADOS DE ITAIPU   : itaipu.dat\n",
    "ARQUIVO C/DEMAND S. BIDDING : bid.dat\n",
    "ARQUIVO C/CARGAS ADICIONAIS : c_adic.dat\n",
    "ARQUIVO C/FATORES DE PERDAS : loss.dat\n",
    "ARQUIVO C/PATAMARES GTMIN   : gtminpat.dat\n",
    "ARQUIVO ENSO 1              : elnino.dat\n",
    "ARQUIVO ENSO 2              : ensoaux.dat\n",
    "ARQUIVO DSVAGUA             : dsvagua.dat\n",
    "ARQUIVO P/PENALID. POR DESV.: penalid.dat\n",
    "ARQUIVO C.GUIA / PENAL.VMINT: curva.dat\n",
    "ARQUIVO AGRUPAMENTO LIVRE   : agrint.dat\n",
    "ARQUIVO DESP. ANTEC. GNL    : adterm.dat\n",
    "ARQUIVO GER. HIDR. MIN      : ghmin.dat\n",
    "ARQUIVO AVERSAO RISCO - SAR : sar.dat\n",
    "ARQUIVO AVERSAO RISCO - CVAR: cvar.dat\n",
    "DADOS DOS RESER.EQ.ENERGIA  : ree.dat\n",
    "ARQUIVO RESTRICOES ELETRICAS: re.dat\n",
    "ARQUIVO DE TECNOLOGIAS      : tecno.dat\n",
    "DADOS DE ABERTURAS          : abertura.dat\n",
    "ARQUIVO DE EMISSOES GEE     : gee.dat\n",
    "ARQUIVO DE RESTRICAO DE GAS : clasgas.dat\n",
]
