class ForwardH:
    """
    Armazena os dados de saída do NEWAVE referentes ao cabeçalho
    dos dados da simulação forward.

    **Parâmetros**

    """

    def __init__(self) -> None:
        pass
