MockREE = [
    " REES X SUBMERCADOS\n",
    " NUM|NOME REES.| SUBM|MES| ANO\n",
    " XXX|XXXXXXXXXX|  XXX| XX|XXXX\n",
    "   1 SUDESTE        1                                                                                                   \n",
    "   6 MADEIRA        1                                                                                                   \n",
    "   7 TPIRES         1                                                                                                   \n",
    "   5 ITAIPU         1                                                                                                   \n",
    "  10 PARANA         1                                                                                                   \n",
    "  12 PRNPANEMA      1                                                                                                   \n",
    "   2 SUL            2                                                                                                   \n",
    "  11 IGUACU         2                                                                                                   \n",
    "   3 NORDESTE       3                                                                                                   \n",
    "   4 NORTE          4                                                                                                   \n",
    "   8 BMONTE         4                                                                                                   \n",
    "   9 MAN-AP         4                                                                                                   \n",
    " 999\n",
    "FICT. INDIVIDUAL.   0000",
    "",
]
