MAX_SERIES_SINTETICAS = 2000
MAX_PATAMARES = 5
NUM_CENARIOS = 2000
MAX_ANOS_ESTUDO = 30
MAX_ANOS_HISTORICO = 120
MAX_CONFIGURACOES = 360
MAX_SUBMERCADOS = 15
MAX_ITERS = 100
MAX_FORWARDS = 300
MAX_CORTES = MAX_ITERS * MAX_FORWARDS
MAX_UHES = 330
MAX_CONJUNTOS_MAQUINAS = 5
MAX_UTES = 300
MAX_UEES = 999
MAX_RES = 100
MAX_REES = 30
MAX_LAG_ADTERM = 2
NUM_CONFIGS_DGER = 100
NUM_VARIAVEIS_MEDIAS = 52
ORDEM_MAX_PARP = 11
MESES = [
    "JANEIRO",
    "FEVEREIRO",
    "MARCO",
    "ABRIL",
    "MAIO",
    "JUNHO",
    "JULHO",
    "AGOSTO",
    "SETEMBRO",
    "OUTUBRO",
    "NOVEMBRO",
    "DEZEMBRO",
]
MESES_DF = [
    "Janeiro",
    "Fevereiro",
    "Março",
    "Abril",
    "Maio",
    "Junho",
    "Julho",
    "Agosto",
    "Setembro",
    "Outubro",
    "Novembro",
    "Dezembro",
]
MESES_ABREV = [
    "JAN",
    "FEV",
    "MAR",
    "ABR",
    "MAI",
    "JUN",
    "JUL",
    "AGO",
    "SET",
    "OUT",
    "NOV",
    "DEZ",
]
MAX_MESES_ESTUDO = MAX_ANOS_ESTUDO * len(MESES)
