"""
inewave
=======

inewave é um módulo em Python para manipulação dos arquivos de entrada
e saída do NEWAVE.
"""

__version__ = "0.0.75"

from . import newave  # noqa
from . import nwlistcf  # noqa
from . import nwlistop  # noqa
