MockBlocoConfUTE = [
    " NUM  NOME           SSIS  U.EXIS CLASSE\n",
    " XXXX XXXXXXXXXXXX   XXXX     XX   XXXX\n",
    "    1 ANGRA 1           1     EX      1\n",
    "   13 ANGRA 2           1     EX     13\n",
    "  211 BAIXADA FLU       1     EX    211\n",
    "   97 CUBATAO           1     EE     97\n",
    "   12 CUIABA G CC       1     EE     12\n",
    "  153 DAIA              1     EE    153\n",
    "   65 DO ATLAN_CSA      1     EE     65\n",
    "  183 DO ATLANTICO      1     EE    183\n",
    "  137 GNA I             1     NE    137\n",
    "  238 GNA P. ACU 3      1     NE    238\n",
    "  155 GOIANIA II        1     EE    155\n",
    "   63 IBIRITE           1     EX     63\n",
    "    2 IGARAPE           1     EE      2\n",
    "   54 JUIZ DE FORA      1     EE     54\n",
    "   15 LINHARES          1     EE     15\n",
    "  236 MARLIM AZUL       1     NE    236\n",
    "  110 N.PIRATINING      1     EE    110\n",
    "  171 NORTEFLU-1        1     EX    171\n",
    "  172 NORTEFLU-2        1     EX    172\n",
    "  173 NORTEFLU-3        1     EE    173\n",
    "  174 NORTEFLU-4        1     EE    174\n",
    "  229 ONCA PINTADA      1     NE    229\n",
    "  182 PALMEIRAS GO      1     EE    182\n",
    "   50 PIRAT.12 G        1     EE     50\n",
    "  230 PREDILECTA        1     NE    230\n",
    "    9 R.SILVEIRA        1     EE      9\n",
    "   62 SEROPEDICA        1     EE     62\n",
    "    4 ST.CRUZ 34        1     EE      4\n",
    "   86 ST.CRUZ NOVA      1     EE     86\n",
    "  146 STA VITORIA       1     NE    146\n",
    "  194 T.NORTE 2         1     EE    194\n",
    "  223 TERMOIRAPE I      1     NE    223\n",
    "   90 TERMOMACAE        1     EE     90\n",
    "   47 TERMORIO          1     EE     47\n",
    "   68 TRES LAGOAS       1     EE     68\n",
    "   49 VIANA             1     EE     49\n",
    "  108 XAVANTES          1     EE    108\n",
    "   48 ARAUCARIA         2     EE     48\n",
    "   37 ARGENTINA 1       2     EX     37\n",
    "   71 ARGENTINA 1B      2     EX     71\n",
    "   38 ARGENTINA 2A      2     EX     38\n",
    "   39 ARGENTINA 2B      2     EX     39\n",
    "   40 ARGENTINA 2C      2     EX     40\n",
    "   59 ARGENTINA 2D      2     EX     59\n",
    "  237 CAMBARA           2     NE    237\n",
    "  156 CANDIOTA 3        2     EE    156\n",
    "   64 CANOAS            2     EE     64\n",
    "  169 CISFRAMA          2     EX    169\n",
    "   28 FIGUEIRA          2     EE     28\n",
    "   26 J.LACERDA A1      2     EE     26\n",
    "   27 J.LACERDA A2      2     EE     27\n",
    "   25 J.LACERDA B       2     EE     25\n",
    "   24 J.LACERDA C       2     EE     24\n",
    "  107 PAMPA SUL         2     EE    107\n",
    "  149 SAO SEPE          2     EX    149\n",
    "   35 URUGUAIANA        2     EE     35\n",
    "  109 ALTOS             3     EE    109\n",
    "  111 ARACATI           3     EE    111\n",
    "  112 BAHIA I           3     EE    112\n",
    "  113 BATURITE          3     EE    113\n",
    "  235 CAMACARI MII      3     NE    235\n",
    "  166 CAMACARI PI       3     EX    166\n",
    "   52 CAMPINA GDE       3     EE     52\n",
    "  117 CAMPO MAIOR       3     EE    117\n",
    "  119 CAUCAIA           3     EE    119\n",
    "  121 CRATO             3     EE    121\n",
    "  125 ENGUIA PECEM      3     EE    125\n",
    "   42 FORTALEZA         3     EE     42\n",
    "   53 GLOBAL I          3     EE     53\n",
    "   55 GLOBAL II         3     EE     55\n",
    "  127 IGUATU            3     EE    127\n",
    "  133 JUAZEIRO N        3     EE    133\n",
    "   57 MARACANAU I       3     EE     57\n",
    "  135 MARAMBAIA         3     EE    135\n",
    "  164 MURICY            3     EE    164\n",
    "  138 NAZARIA           3     EE    138\n",
    "  167 P. PECEM I        3     EE    167\n",
    "  163 P. PECEM II       3     EX    163\n",
    "  224 P. SERGIPE I      3     NE    224\n",
    "  160 PAU FERRO I       3     EE    160\n",
    "  162 PECEM II          3     NE    162\n",
    "   98 PERNAMBU_III      3     EE     98\n",
    "  144 PETROLINA         3     EE    144\n",
    "  151 POTIGUAR          3     EE    151\n",
    "  161 POTIGUAR III      3     EE    161\n",
    "  147 PROSPERIDADE      3     EE    147\n",
    "  170 SUAPE II          3     EE    170\n",
    "  221 SYKUE I           3     EX    221\n",
    "   43 TERMOBAHIA        3     EE     43\n",
    "  152 TERMOCABO         3     EE    152\n",
    "   74 TERMOCAMACAR      3     EE     74\n",
    "   58 TERMOCEARA        3     EE     58\n",
    "  159 TERMOMANAUS       3     EE    159\n",
    "   67 TERMONE           3     EE     67\n",
    "   69 TERMOPB           3     EE     69\n",
    "   96 TERMOPE           3     EE     96\n",
    "   83 VALE DO ACU       3     EE     83\n",
    "  201 APARECIDA         4     EE    201\n",
    "  203 C. ROCHA          4     EE    203\n",
    "  318 FLORES LT1        4     EE    318\n",
    "  319 FLORES LT2        4     EE    319\n",
    "   73 GERAMAR I         4     EE     73\n",
    "   70 GERAMAR II        4     EE     70\n",
    "  317 IRANDUBA          4     EE    317\n",
    "  204 JARAQUI           4     EE    204\n",
    "  205 MANAUARA          4     EE    205\n",
    "  212 MARANHAO III      4     EE    212\n",
    "  436 MARAN IV L22      4     NE    436\n",
    "  437 MARAN IV L7       4     NE    437\n",
    "   36 MARANHAO IV       4     EE     36\n",
    "  421 MARAN V L22       4     NE    421   \n",
    "  422 MARAN V L7        4     NE    422   \n",
    "   21 MARANHAO V        4     EE     21\n",
    "  140 MAUA 3            4     EE    140\n",
    "  206 MAUA B3           4     EE    206\n",
    "  141 MAUA B4           4     EE    141\n",
    "  446 N.VEN 2 L22       4     NE    446 \n",
    "  447 N.VEN 2 L7        4     NE    447\n",
    "   46 N.VENECIA 2       4     EE     46\n",
    "  228 O. CANOAS 1       4     NE    228\n",
    "  116 PARNAIBA IV       4     EE    116\n",
    "  239 PARNAIBA V        4     NE    239\n",
    "  207 PONTA NEGRA       4     EE    207\n",
    "  176 PORTO ITAQUI      4     EE    176\n",
    "  209 TAMBAQUI          4     EE    209\n",
    "",
]
